#! /bin/sh
#
# Name: rootsize.sh
# Author: SpinetiX S.A.
# Copyright: 2008 SpinetiX S.A.
# License: 2008 (c) SpinetiX S.A. This file is licensed
#          under the terms of the GNU General Public License version 2.
#          This program is licensed "as is" without any warranty of any
#          kind, whether express or implied.
#
# Copyright 1999-2003 MontaVista Software, Inc.
# Copyright 2002, 2003, 2004 Sony Corporation
# Copyright 2002, 2003, 2004 Matsushita Electric Industrial Co., Ltd.
#
### BEGIN INIT INFO
# Required-Start:
# Required-Stop:
# Should-Start:
# Should-Stop:
# Default-Start:
# Default-Stop:
# Short-Description: checks root fs free size
# Description: checks root fs free size and cleans files if necessary
### END INIT INFO
# chkconfig: S 15 0

# NOTE: this should run as early as possible after root fs has been
# mounted rw.

# Init script information
INIT_NAME=rootsize
DESC="root filesystem free size check"

# Load init script configuration
[ -f /etc/default/$INIT_NAME ] && . /etc/default/$INIT_NAME

# Source the init script functions
. /etc/init.d/init-functions

case "$1" in

    start)
	# Get minimum number of free blocks and current free blocks
	# (block counts returned by stat use the fs native block size)
	# NOTE: our stat version does not support %S but %s returns
	# identical information for our filesystems
	set -- $(stat -f -c '%f %s' /)
	nr_free="$1"
	block_sz="$2"
	if [ "$block_sz" -ge 1024 ]; then
	    let "min_free = ROOT_MIN_FREE_KB / ( block_sz / 1024 )"
	else
	    let "min_free = ROOT_MIN_FREE_KB * ( 1024 / block_sz )"
	fi

	# Check free blocks
	if [ "$nr_free" -lt "$min_free" ]; then
	    log_warning_msg "Less than $ROOT_MIN_FREE_KB KBytes free in root fs, cleaning up files"
	    # Remove files from /var/tmp right away
	    find /var/tmp -type f -print0 | xargs -0r rm -f --
	fi

	# recheck available space
	nr_free="$(stat -f -c '%f' /)"
	if [ "$nr_free" -lt "$min_free" ]; then
	    log_warning_msg "Clearing large files from /var/log to free space"
	    find /var/log -mindepth 1 -depth -type f -size +5000k -exec cp /dev/null \{\} \;
	fi

	# recheck available space
	nr_free="$(stat -f -c '%f' /)"
	if [ "$nr_free" -lt "$min_free" ]; then
	    log_warning_msg "Removing files from /var/cache to free space"
	    find /var/cache -type f -print0 | xargs -0r rm -f --
	fi

	# recheck available space
	nr_free="$(stat -f -c '%f' /)"
	if [ "$nr_free" -lt "$min_free" ]; then
	    log_warning_msg "Clearing all files from /var/log to free space"
	    find /var/log -mindepth 1 -depth -type f -exec cp /dev/null \{\} \;
	fi

	# recheck available space
	nr_free="$(stat -f -c '%f' /)"
	if [ "$nr_free" -lt "$min_free" ]; then
	    log_failure_msg "Still less than $ROOT_MIN_FREE_KB KBytes free after cleaning!"
	fi
	;;

    stop)
	;;
esac
